﻿using System.Collections.Generic;

namespace HeuristicLab.Algorithms.PushGP.Data
{
    public class Configuration
    {
        public bool IsBooleanStackEnabled { get; set; } = true;
        public bool IsIntegerStackEnabled { get; set; } = true;
        public bool IsFloatStackEnabled { get; set; } = true;
        public bool IsCodeStackEnabled { get; set; } = true;
        public bool IsNameStackEnabled { get; set; } = true;

        public IList<OpCode> AllowedInstructions { get; set; }

        /// <summary>
        /// This is the maximum allowed number of "executions" in a single top-level call to the interpreter.
        /// The execution of a single Push instruction counts as one execution, as does the processing of a single literal,
        /// as does the descent into one layer of parentheses (that is, the processing of the "(" counts as one execution).
        /// When this limit is exceeded the interpreter aborts immediately, leaving its stacks in the states they were in prior
        /// to the abort (so they may still be examined by a calling program). Whether or not this counts as an "abnormal" termination
        /// is up to the calling program.
        /// </summary>
        public uint EvalpushLimit { get; set; } = int.MaxValue;

        /// <summary>
        /// This is the maximum size of an item on the CODE stack, expressed as a number of points.
        /// A point is an instruction, a literal, or a pair of parentheses. Any instruction that would cause this limit to be exceeded
        /// should instead act as a NOOP, leaving all stacks in the states that they were in before the execution of the instruction.
        /// </summary>
        public uint MaxPointsInProgram { get; set; } = int.MaxValue;

        public bool TopLevelPushCode { get; set; } = true;
    }
}
